<?php
if ( ! class_exists( 'IPOptz_Core' )) 
{
	/**
	 * Core class
	 *
	 * @package wp-swift-page
	 * @author InstaProduct
	 * @link http://insta-product.com/
	 */

	class IPOptz_Core
	{
		//{{ init

		/**
		 * Run core class function
		 *
		 * @return void
		 */
		public static function init()
		{
			//Remove header tags
			add_action( 'after_setup_theme', array( __CLASS__, 'initHeaderTags' ) );

			//Editing style registration
			add_action( 'wp_print_styles', array( __CLASS__, 'registerStyle' ), 100 );

			if ( ! is_admin() ) 
			{
				self::initScript();
				self::initStyle();
				self::initGoogleLibs();
			}
		}

		//}}
		//{{ initHeaderTags

		/**
		 * Run header tags function
		 *
		 * @return void
		 */
		public static function initHeaderTags()
		{
			//Remove RSD links
			$option = get_option( '__ipoptz_remove_rsd_link', 'false' );
			if ( $option == 'true' ) {
				remove_action( 'wp_head', 'rsd_link' );
			}

			//Adjacent Posts links
			$option = get_option( '__ipoptz_remove_adjacent_links', 'false' );
			if ( $option == 'true' ) {
				remove_action( 'wp_head', 'adjacent_posts_rel_link_wp_head' );
			}

			//Remove Windows Live Writer Manifest Link
			$option = get_option( '__ipoptz_remove_windows_manifest', 'false' );
			if ( $option == 'true' ) {
				remove_action( 'wp_head', 'wlwmanifest_link' );
			}

			//Remove WordPress Shortlinks from WP Head
			$option = get_option( '__ipoptz_remove_wp_shortlink', 'false' );
			if ( $option == 'true' ) {
				remove_action( 'wp_head', 'wp_shortlink_wp_head' );
			}

			//Remove WP Generator
			$option = get_option( '__ipoptz_remove_wp_version', 'false' );
			if ( $option == 'true' ) {
				remove_action( 'wp_head', 'wp_generator' );
			}

			//Remove all feeds
			$option = get_option( '__ipoptz_remove_rss_links', 'false' );
			if ( $option == 'true' ) {
				remove_action( 'wp_head', 'feed_links_extra', 3 );
				remove_action( 'wp_head', 'feed_links', 2 );
			}

		}

		//}}
		//{{ initScript

		/**
		 * Run script function
		 *
		 * @return void
		 */
		public static function initScript()
		{
			$scriptToFooter = get_option( '__ipoptz_scripts_to_footer', 'false' );
			$scriptMerge    = get_option( '__ipoptz_scripts_merge', 'false' );
			$styleToFooter  = get_option( '__ipoptz_styles_to_footer', 'false' );
			$styleMerge     = get_option( '__ipoptz_styles_merge', 'false' );
			//
			//Replace print script in footer function
			if ( $scriptToFooter == 'true' || $scriptMerge == 'true' ||
					 $styleToFooter == 'true' || $styleMerge == 'true'
			) {
				remove_action( 'wp_footer', 'wp_print_footer_scripts', 20 );
				add_action( 'wp_footer', array( __CLASS__, 'printAssetsFooter' ), 20 );
			}

			//
			//Remove print script in head
			if ( $scriptToFooter == 'true' || $scriptMerge == 'false' ) 
			{
				remove_action( 'wp_head', 'wp_print_scripts' );
				remove_action( 'wp_head', 'wp_print_head_scripts', 9 );
			}

			//
			// Defer Parsing
			$option = get_option( '__ipoptz_defer_parsing', 'false' );
			if ( $option == 'true' ) {
				add_filter( 'clean_url', array( __CLASS__, 'doDeferParsing' ), 11, 1 );
			}

			//
			// Remove Query Strings
			$option = get_option( '__ipoptz_remove_script_query', 'false' );
			if ( $option == 'true' ) {
				add_filter( 'script_loader_src', 
										array( __CLASS__, 'removeQueryArgs' ), 
										15, 1 
									);
			}

			//
			// Remove Query Strings
			$option = get_option( '__ipoptz_remove_wp_emoji', 'false' );
			if ( $option == 'true' ) {
				remove_action( 'wp_head', 'print_emoji_detection_script', 7 );
				remove_action( 'wp_print_styles', 'print_emoji_styles' );
			}
		}

		//}}
		//{{ initStyle

		/**
		 * Run style function
		 *
		 * @return void
		 */
		public static function initStyle()
		{
			//
			// Move all styles to footer
			$toFooter = get_option( '__ipoptz_styles_to_footer', 'false' );
			$merge    = get_option( '__ipoptz_styles_merge', 'false' );
			if ( $toFooter == 'true' && $merge == 'false' ) {
				self::_moveStylesToFooter();
			}

			//
			// Combine all styles in one file
			if ( $merge == 'true' ) {
				self::_combineStyles();
			}

			if ( $merge == 'false' && $toFooter == false ) {
				self::_prepareMoveStylesToFooter();
			}

			//
			// Remove Query Strings
			$option = get_option( '__ipoptz_remove_style_query', 'false' );
			if ( $option == 'true' ) {
				add_filter( 'style_loader_src', 
										array( __CLASS__, 'removeQueryArgs' ), 
										15, 1 
									);
			}
		}

		//}}
		//{{ initGoogleLibs

		/**
		 * Run google libs replacement
		 *
		 * @return void
		 */
		public static function initGoogleLibs()
		{
			$loadGoogle  = get_option( '__ipoptz_load_from_google', 'false' );
			$scriptMerge = get_option( '__ipoptz_scripts_merge', 'false' );
			if ( $loadGoogle == 'true' && $scriptMerge == 'false' ) {
				IPOptz_GoogleLib::init();
			}
		}

		//}}
		//{{ registerStyle

		/**
		 * Do defer parsing
		 *
		 * @return void
		 */
		public static function registerStyle()
		{
			//
			// Remove dashicon
			$option = get_option( '__ipoptz_remove_dashicon', 'false' );
			if ( $option == 'true' ) {
				wp_deregister_style( 'dashicons' );
			}
			//
			// Remove font awesome
			$option = get_option( '__ipoptz_remove_font_awesome', 'false' );
			if ( $option == 'true' ) {
				wp_deregister_style( 'font-awesome-css' );
				wp_deregister_style( 'font-awesome' );
			}
		}

		//}}
		//{{ doDeferParsing

		/**
		 * Do defer parsing
		 *
		 * @return void
		 */
		public static function doDeferParsing( $url )
		{
			if ( strpos( $url, '.js' ) === FALSE ) {
				return $url;
			}
			return "$url' defer='defer";
		}

		//}}
		//{{ removeQueryArgs

		/**
		 * Do defer parsing
		 *
		 * @return void
		 */
		public static function removeQueryArgs( $url )
		{
			$url = remove_query_arg( array(
				'v',
				'ver',
				'version',
				'rev'
			), $url );

			return $url;
		}

		//}}
		//{{ printAssetsFooter

		/**
		 * Do defer parsing
		 *
		 * @return void
		 */
		public static function printAssetsFooter()
		{
			//
			// Combine all scripts in one file
			$scriptMerge = get_option( '__ipoptz_scripts_merge', 'false' );
			if ( $scriptMerge == 'true' ) {
				self::printCombineScript();
			}
			//
			// Combine all styles in one file
			$styleMerge = get_option( '__ipoptz_styles_merge', 'false' );
			if ( $styleMerge == 'true' ) {
				self::printCombineStyle();
			}

			//Print scripts
			if ( $scriptMerge == 'false' ) {
				self::printScripts();
			}

			//Print styles
			$toFooter = get_option( '__ipoptz_styles_to_footer', 'false' );
			if ( $styleMerge == 'false' && $toFooter == false ) {
				self::printStyles();
			}

		}

		//}}
		//{{ printCombineScript

		/**
		 * Print Combining all scripts in one file
		 *
		 * @return void
		 */
		public static function printCombineScript()
		{
			global $post;

			$defer  = '';
			$option = get_option( '__ipoptz_defer_parsing', 'false' );
			if ( $option == 'true' ) {
				$defer = ' defer="defer"';
			}
			$src = get_home_url() . '/ipoptz-request/getscript';
			if ( is_page() && ! empty( $post )) {
				$src .= '/' . $post->ID . '/';
			} else {
				$src .= '/';
			}
			echo '<script type="text/javascript" id="ipoptz-script-combine" src="' . $src . '"' . $defer . '></script>';
		}

		//}}
		//{{ _moveStylesToFooter

		/**
		 * Move all styles to footer
		 *
		 * @return void
		 */
		protected static function _moveStylesToFooter()
		{
			global $wp_filter;

			remove_action( 'wp_head', 'wp_print_styles', 8 );

			self::_prepareMoveStylesToFooter();

			//Add footer hook
			add_action( 'wp_footer', array( __CLASS__, 'printStyles' ), 8 );
		}

		//}}
		//{{ _prepareMoveStylesToFooter

		/**
		 * Move all styles to footer
		 *
		 * @return void
		 */
		protected static function _prepareMoveStylesToFooter()
		{
			global $wp_filter;

			//Try remove action from plugin
			$hooks = $wp_filter['wp_head'];
			if ( ! empty( $hooks )) 
			{
				foreach( $hooks as $index => $hook )
				{
					foreach( $hook as $data )
					{
						if ( is_array( $data['function'] ))
						{
							$classname  = $data['function'][0];
							$methodname = $data['function'][1];
							if ( class_exists( $classname ) && $methodname = 'publicStyle' ) 
							{
								remove_action( 'wp_head', $data['function'] );
								add_action( 'wp_footer', $data['function'] );
							}
						}
					}
				}
			}
		}

		//}}
		//{{ _combineStyles

		/**
		 * Combine all scripts in one file
		 *
		 * @return void
		 */
		protected static function _combineStyles()
		{
			global $wp_filter;

			remove_action( 'wp_head', 'wp_print_styles', 8 );
		}

		//}}
		//{{ printStyles

		/**
		 * Print all styles
		 *
		 * @return void
		 */
		public static function printStyles()
		{
			global $wp_styles;

			$handles = false;
			do_action( 'wp_print_styles' );

			if ( ! ( $wp_styles instanceof WP_Styles ) ) {
				if ( ! $handles ) {
					return array();
				}
			}

			//Remove unnecessery styles
			self::_removeGlobalStyles();

			return wp_styles()->do_items( $handles );
		}

		//}}
		//{{ printScripts

		/**
		 * Print all scripts
		 *
		 * @return void
		 */
		public static function printScripts()
		{
			global $wp_scripts;

			$handles = false;
			do_action( 'wp_print_scripts' );

			if ( ! ( $wp_scripts instanceof WP_Scripts ) ) {
				if ( ! $handles ) {
					return array();
				}
			}

			//Remove unnecessery scripts
			self::_removeGlobalScripts();

			return wp_scripts()->do_items( $handles );
		}

		//}}
		//{{ printCombineStyle

		/**
		 * Print Combining all styles in one file
		 *
		 * @return void
		 */
		public static function printCombineStyle()
		{
			global $post;

			$href = get_home_url() . '/ipoptz-request/getstyle';
			if ( is_page() && ! empty( $post )) {
				$href .= '/' . $post->ID . '/';
			} else {
				$href .= '/';
			}
			echo '<link rel="stylesheet" id="ipoptz-style-combine" href="'.$href.'"></script>';
		}

		//}}
		//{{ _removeGlobalStyles

		/**
		 * Remove unnecessery styles
		 *
		 * @return void
		 */
		protected static function _removeGlobalStyles()
		{
			$regstyles = self::getRegisteredStyles();
			$excludes  = self::getRemovedScripts();
			if ( ! empty( $optstyles ) && ! empty( $excludes )) 
			{
				foreach( $regstyles as $name => $url )
				{
					if ( in_array( $name, $excludes )) {
						wp_deregister_style( $name );
					}
				}

			}
		}

		//}}
		//{{ _removeGlobalScripts

		/**
		 * Remove unnecessery scripts
		 *
		 * @return void
		 */
		protected static function _removeGlobalScripts()
		{
			$regscripts = self::getRegisteredScripts();
			$excludes   = self::getRemovedScripts();
			if ( ! empty( $optscripts ) && ! empty( $excludes )) 
			{
				foreach( $regscripts as $name => $url )
				{
					if ( in_array( $name, $excludes )) {
						wp_deregister_script( $name );
					}
				}

			}
		}

		//}}
		//{{ getRemovedStyles

		/**
		 * Get removed styles
		 *
		 * @param int $postId 
		 * @return void
		 */
		public static function getRemovedScripts( $postId = false )
		{
			$includes   = array();
			$optscripts = array();
			if ( ! empty( $postId )) {
				$optscripts = get_post_meta( $postId, '__ipoptz_page_scripts', true );
			}
			if ( empty( $optscripts )) {
				$optscripts = get_option( '__ipoptz_scripts' );
			}
			if ( ! empty( $optscripts )) 
			{
				$includes = explode( ',', $optscripts );
				foreach( $includes as $key => $name )
				{
					$name = trim( $name );
					$name = trim( $name, ',' );
					$includes[$key] = $name;
				}
			}
			return $includes;
		}

		//}}
		//{{ getRemovedStyles

		/**
		 * Get removed styles
		 *
		 * @param int $postId 
		 * @return void
		 */
		public static function getRemovedStyles( $postId = false )
		{
			$includes  = array();
			$optstyles = array();
			if ( ! empty( $postId )) {
				$optstyles = get_post_meta( $postId, '__ipoptz_page_styles', true );
			}
			if ( empty( $optstyles )) {
				$optstyles = get_option( '__ipoptz_styles' );
			}

			if ( ! empty( $optstyles )) 
			{
				$includes = explode( ',', $optstyles );
				foreach( $includes as $key => $name )
				{
					$name = trim( $name );
					$name = trim( $name, ',' );
					$includes[$key] = $name;
				}
			}
			return $includes;
		}

    //}}
    //{{ getRegisteredStyles

    /**
     * Get registered styles
     *
     * @return array
     */
    public static function getRegisteredStyles()
    {
      global $wp_styles;

      $styles = array();
      $handles = array();
      if ( ! empty( $wp_styles->registered ))
      {
        foreach( $wp_styles->registered as $obj ) 
        {
          $name = $obj->handle;
          $styles[$name] = $obj->src;
        }
      }

      $results = array();
      $handles = array_merge( $wp_styles->queue );
      array_values( $handles );

      if ( ! empty( $handles ))
      {
        foreach( $handles as $handle ) 
        {
        	if ( ! empty( $styles[$handle] ) ) {
          	$results[$handle] = $styles[$handle];
        	}
        }
      }
      return $results;
    }

    //}}
    //{{ getRegisteredScripts

    /**
     * Get registered scripts
     *
     * @return array
     */
    public static function getRegisteredScripts()
    {
      global $wp_scripts;

      $scripts = array();
      $handles = array();
      if ( ! empty( $wp_scripts->registered ))
      {
        foreach( $wp_scripts->registered as $obj ) 
        {
          $name = $obj->handle;
          $scripts[$name] = $obj->src;
        }
      }

      $results = array();
      $handles = array_merge( $wp_scripts->queue );
      array_values( $handles );

      if ( ! empty( $handles ))
      {
        foreach( $handles as $handle ) 
        {
        	if ( ! empty( $scripts[$handle] ) ) {
          	$results[$handle] = $scripts[$handle];
        	}
        }
      }
      return $results;
    }

    //}}
    //{{ getHtaccessEtag

    /**
     * Get .htaccess code for disabled ETag
     *
     * @return string
     */
    public static function getHtaccessEtag()
    {
      $code = '';
      $code .= "## DISABLED ETAG ##\n";
      $code .= "Header unset Pragma\n";
      $code .= "FileETag None\n";
      $code .= "Header unset ETag\n";
      $code .= "\n";

      return $code;
    }

    //}}
    //{{ getHtaccessExpires

    /**
     * Get .htaccess code for set browser expires
     *
     * @param  boolean $preview
     * @return string
     */
    public static function getHtaccessExpires( $preview = false )
    {
    	if ( $preview ) {
	    	$lt = '&lt';
	    	$gt = '&gt';
    	} else {
	    	$lt = '<';
	    	$gt = '>';
    	}

      $code = '';
      $code .= "## EXPIRES CACHING ##\n";
      $code .= $lt . 'IfModule mod_expires.c' . $gt . "\n";
      $code .= "ExpiresActive On\n";
      $code .= 'ExpiresByType image/jpg "access 1 year"' . "\n";
      $code .= 'ExpiresByType image/jpeg "access 1 year"' . "\n";
      $code .= 'ExpiresByType image/gif "access 1 year"' . "\n";
      $code .= 'ExpiresByType image/png "access 1 year"' . "\n";
      $code .= 'ExpiresByType text/css "access 1 month"' . "\n";
      $code .= 'ExpiresByType text/html "access 1 month"' . "\n";
      $code .= 'ExpiresByType application/pdf "access 1 month"' . "\n";
      $code .= 'ExpiresByType text/x-javascript "access 1 month"' . "\n";
      $code .= 'ExpiresByType application/x-shockwave-flash "access 1 month"' . "\n";
      $code .= 'ExpiresByType image/x-icon "access 1 year"' . "\n";
      $code .= 'ExpiresDefault "access 1 month"' . "\n";
      $code .= $lt . '/IfModule' . $gt . "\n";
      $code .= "\n";

      return $code;
    }

    //}}
    //{{ getKeepAlive

    /**
     * Get .htaccess code for set browser expires
     *
     * @param  boolean $preview
     * @return string
     */
    public static function getKeepAlive( $preview = false )
    {
    	if ( $preview ) {
	    	$lt = '&lt';
	    	$gt = '&gt';
    	} else {
	    	$lt = '<';
	    	$gt = '>';
    	}
    	
      $code = '';
      $code .= "## HTTP KEEP-ALIVE ##\n";
      $code .= $lt . 'IfModule mod_headers.c' . $gt . "\n";
      $code .= "Header set Connection keep-alive\n";
      $code .= $lt . '/IfModule' . $gt . "\n";
      $code .= "\n";

      return $code;
    }

    //}}
    //{{ getHtaccessGzip

    /**
     * Get .htaccess code for enable GZIP compression
     *
     * @param  boolean $preview
     * @return string
     */
    public static function getHtaccessGzip( $preview = false )
    {
    	if ( $preview ) {
	    	$lt = '&lt';
	    	$gt = '&gt';
    	} else {
	    	$lt = '<';
	    	$gt = '>';
    	}

      $code = '';
      $code .= "## ENABLE GZIP COMPRESSION ##\n";
      $code .= $lt . 'FilesMatch "\\.(js|css|html|htm|php|xml)$"' . $gt . "\n";
      $code .= 'SetOutputFilter DEFLATE' . "\n";
      $code .= $lt . '/FilesMatch' . $gt . "\n";
      $code .= "\n";
      $code .= $lt . 'IfModule mod_gzip.c' . $gt . "\n";
      $code .= 'mod_gzip_on Yes' . "\n";
      $code .= 'mod_gzip_dechunk Yes' . "\n";
      $code .= 'mod_gzip_item_include file \.(html?|txt|css|js|php|pl)$' . "\n";
      $code .= 'mod_gzip_item_include handler ^cgi-script$' . "\n";
      $code .= 'mod_gzip_item_include mime ^text/.*' . "\n";
      $code .= 'mod_gzip_item_include mime ^application/x-javascript.*' . "\n";
      $code .= 'mod_gzip_item_exclude mime ^image/.*' . "\n";
      $code .= 'mod_gzip_item_exclude rspheader ^Content-Encoding:.*gzip.*' . "\n";
      $code .= $lt . '/IfModule' . $gt . "\n";
      $code .= "\n";

      return $code;
    }
		//}}
	}
}