<?php
if ( ! class_exists( 'IPOptz_Viewer' ))
{
  /**
   * Viewer class
   *
   * @package wp-swift-page
   * @author InstaProduct
   * @link http://insta-product.com/
   */

  class IPOptz_Viewer
  {
    //{{ dashboard

    /**
     * Dashboard viewer
     *
     * @return void
     */
    public function dashboard()
    {
      $this->pageId = IPOPTZ_PAGENAME;

      $attachments = get_posts( array(
        'post_type'      => 'attachment',
        'posts_per_page' => -1,
        'post_mime_type' => 'image/jpeg,image/jpg,image/gif,image/png'
      ) );
      $this->allImagesCount = count( $attachments );

      //
      //Get GTMatrix data
      //
      
      $loadTime = 0;
      $request  = 0;
      $pageSize = 0;
      $gtScore  = 0;
      $ysScore  = 0;
      $status   = false;
      $results  = array();

      $gtmetrixApiKey = get_option( '__ipoptz_gtmetrix_api_key' );
      $gtmetrixEmail  = get_option( '__ipoptz_gtmetrix_email' );
      if ( ! empty( $gtmetrixApiKey ) && ! empty( $gtmetrixEmail ))
      {
        $homeUrl  = get_home_url();
        $test     = new Services_WTF_Test( $gtmetrixEmail, $gtmetrixApiKey );
        $testid = $test->test(array(
            'url' => $homeUrl
        ));
        $test->get_results();
        
        $results = $test->results();

        if ( ! empty( $results['page_load_time'] ) ) {
          $loadTime = round( $results['page_load_time']/1000, 2 );
        }
        if ( ! empty( $results['page_bytes'] ) ) {
          $pageSize = round( $results['page_bytes']/1000 );
        }
        if ( ! empty( $results['page_elements'] ) ) {
          $request = $results['page_elements'];
        }
        if ( ! empty( $results['pagespeed_score'] ) ) {
          $gtScore = $results['pagespeed_score'];
        }
        if ( ! empty( $results['yslow_score'] ) ) {
          $ysScore = $results['yslow_score'];
        }
        $status = true;
      }
      $this->gtmetrix   = $status;
      $this->gtResults  = $results;
      $this->gtRequest  = $request;
      $this->gtLoadTime = $loadTime;
      $this->gtPageSize = $pageSize;
      $this->gtScore    = $gtScore;
      $this->ysScore    = $ysScore;

      $this->view( 'admin/dashboard' );
    }

    //}}
    //{{ script

    /**
     * Script viewer
     *
     * @return void
     */
    public function script()
    {
      $this->pageId        = IPOPTZ_PAGENAME . '-script';
      $this->pluginScripts = $this->_getRegisteredScripts();

      $this->view( 'admin/script' );
    }

    //}}
    //{{ style

    /**
     * Style viewer
     *
     * @return void
     */
    public function style()
    {
      $this->pageId       = IPOPTZ_PAGENAME . '-style';
      $this->pluginStyles = $this->_getRegisteredStyles();

      $this->view( 'admin/style' );
    }

    //}}
    //{{ general

    /**
     * General viewer
     *
     * @return void
     */
    public function general()
    {
      $path    = ABSPATH . '.htaccess';
      $handle  = fopen( $path, "r" );
      $content = fread( $handle, filesize($path) );

      $this->pageId           = IPOPTZ_PAGENAME . '-general';
      $this->htaccessPath     = $path;
      $this->htaccessWritable = is_writable( $path );
      $this->htaccessContent  = $content;

      $this->view( 'admin/general' );
    }

    //}}
    //{{ image

    /**
     * General viewer
     *
     * @return void
     */
    public function image()
    {
      //Including script
      wp_enqueue_media();

      $this->pageId = IPOPTZ_PAGENAME . '-image';

      $this->view( 'admin/image' );
    }

    //}}
    //{{ setting

    /**
     * Setting page viewer
     *
     * @return void
     */
    public function setting()
    {
      $this->pageId = IPOPTZ_PAGENAME . '-setting';

      $this->view( 'admin/setting' );
    }

    //}}
    //{{ view

    /**
     * Get admin view file
     *
     * @param string $filename
     * @return void
     */
    public function view( $filename )
    {
      $path  = IPOPTZ_VIEW;
      $path .= strtolower( $filename ) . '.php';
      if ( file_exists( $path )) {
        include( $path );
      }
    }

    //}}
    //{{ getPercentClass

    /**
     * Get percent class
     *
     * @param int $percent
     * @return void
     */
    public function getPercentClass( $percent )
    {
      if ( $percent > 60 ) {
        $class = 'ipoptz-stat-success';
      }
      elseif ( $percent > 31 && $percent < 59 ) {
        $class = 'ipoptz-stat-warning';
      }
      else {
        $class = 'ipoptz-stat-danger';
      }
      return $class;
    }

    //}}
    //{{ _getRegisteredScripts

    /**
     * Get registered scripts
     *
     * @return array
     */
    protected function _getRegisteredScripts()
    {
      $scripts = get_option( '__ipoptz_registered_scripts' );
      $scripts = maybe_unserialize( $scripts );
      return $scripts;
    }

    //}}
    //{{ _getRegisteredStyles

    /**
     * Get registered styles
     *
     * @return array
     */
    protected function _getRegisteredStyles()
    {
      $styles = get_option( '__ipoptz_registered_styles' );
      $styles = maybe_unserialize( $styles );
      return $styles;
    }
    //}}
  }
}
