<?php
if ( ! class_exists( 'IPOptz_Request' )) 
{
  /**
   * Request handler
   *
   * @package wp-swift-page
   * @author InstaProduct
   * @link http://insta-product.com/
   */

  class IPOptz_Request
  {
    /**
     * Take action name
     *
     * @var string
     */
    protected $_action;

    //}}
    //{{ __construct

    /**
     * Class constructor
     *
     * @param  string $action
     * @return void
     */
    public function __construct( $action )
    {
      $this->_action = $action;
    }

    //}}
    //{{ render

    /**
     * Render request
     *
     * @return void
     */
    public function render()
    {
      if ( empty( $this->_action )) {
        $this->_show404();
        return false;
      }
      else 
      {
        $action = lcfirst( $this->_action );
        if ( ! method_exists( $this, $action )) {
          $this->_show404();
          return false;
        }
        else
        {
          $this->$action();
          exit();
        }
      }
    }

    //}}
    //{{ getscript

    /**
     * Get script
     *
     * @return void
     */
    public function getscript()
    {
      global $wp_scripts, $concatenate_scripts, $wp_query;

      wp_enqueue_scripts();

      if ( ! ( $wp_scripts instanceof WP_Scripts ) ) {
        return array(); // No need to run if not instantiated.
      }
      script_concat_settings();
      $wp_scripts->do_concat = $concatenate_scripts;

      $handles = $wp_scripts->queue;
      $wp_scripts->all_deps( $handles );

      //Set header
      header( 'Content-type: text/javascript' );

      foreach ( $wp_scripts->to_do as $key => $handle ) 
      {
        if ( ! in_array($handle, $wp_scripts->done, true) && 
               isset($wp_scripts->registered[$handle]) ) 
        {
          $obj = $wp_scripts->registered[$handle];
          if ( empty( $obj->src )) {
            continue;
          }

          $postId = 0;
          if ( isset( $wp_query->query['ipoptz_post'] )) {
            $postId = absint( $wp_query->query['ipoptz_post'] );
          }
          $excludes = IPOptz_Core::getRemovedScripts( $postId );
          if ( ! empty( $excludes ) && in_array($handle, $excludes)) {
            continue;
          }

          $path = false;
          if ( strpos( $obj->src, '/wp-includes' ) === 0 ) {
            $path = ABSPATH . trim( $obj->src, '/' );
            $this->_readFileJs( $path );
          }
          else
          {
            $home  = get_home_url();
            $partsHome = parse_url( $home );
            $partsSrc  = parse_url( $obj->src );
            
            if ( $partsHome['host'] == $partsSrc['host'] ) 
            {
              $pathFile = str_replace( $partsHome['path'], '', $partsSrc['path'] );
              $pathFile = trim( $pathFile, '/' );
              $path = ABSPATH . $pathFile;
              $this->_readFileJs( $path );
            }
            else
            {
              $script = file_get_contents( $obj->src );
              if ( $script )
              {
                echo $this->_minifyJs( $script );
                echo "\n\n";
              }
            }
          }

          unset( $wp_scripts->to_do[$key] );
        }
      }
    }

    //}}
    //{{ getstyle

    /**
     * Get style
     *
     * @return void
     */
    public function getstyle()
    {
      global $wp_styles;

      wp_enqueue_scripts();

      if ( ! ( $wp_styles instanceof WP_Styles ) ) {
        return array(); // No need to run if not instantiated.
      }

      $handles = $wp_styles->queue;
      $wp_styles->all_deps( $handles );

      //Set header
      header( 'Content-type: text/css' );

      foreach ( $wp_styles->to_do as $key => $handle ) 
      {
        if ( ! in_array($handle, $wp_styles->done, true) && 
               isset($wp_styles->registered[$handle]) ) 
        {
          $obj = $wp_styles->registered[$handle];
          if ( empty( $obj->src )) {
            continue;
          }

          $postId = 0;
          if ( isset( $wp_query->query['ipoptz_post'] )) {
            $postId = absint( $wp_query->query['ipoptz_post'] );
          }
          $excludes = IPOptz_Core::getRemovedStyles( $postId );
          
          if ( ! empty( $excludes ) && in_array($handle, $excludes)) {
            continue;
          }

          $path = false;
          if ( strpos( $obj->src, '/wp-includes' ) === 0 ) {
            $path = ABSPATH . trim( $obj->src, '/' );
            $this->_readFileCss( $path );
          }
          else
          {
            $home  = get_home_url();
            $partsHome = parse_url( $home );
            $partsSrc  = parse_url( $obj->src );

            
            if ( $partsHome['host'] == $partsSrc['host'] ) 
            {
              $pathFile = str_replace( $partsHome['path'], '', $partsSrc['path'] );
              $pathFile = trim( $pathFile, '/' );
              if ( strpos( $pathFile, '-content' ) === 0 ) {
                $pathFile = 'wp' . $pathFile;
              }
              $path = ABSPATH . $pathFile;
              $this->_readFileCss( $path );
            }
            else
            {
              $script = file_get_contents( $obj->src );
              if ( $script )
              {
                echo $this->_minifyCss( $script );
                echo "\n\n";
              }
            }
          }

          unset( $wp_styles->to_do[$key] );
        }
      }
    }

    //}}
    //{{ _readFileJs

    /**
     * Read file javascript
     *
     * @param  string $file
     * @return void
     */
    protected function _readFileJs( $file )
    {
      if ( file_exists( $file ) && filesize($file) > 0 ) 
      {
        $handle   = fopen( $file, "r" );
        $contents = fread( $handle, filesize($file) );
        fclose( $handle );

        $contents = $this->_minifyJs( $contents );
        echo "$contents\n\n";
      }
    }

    //}}
    //{{ _readFileCss

    /**
     * Read file css
     *
     * @param  string $file
     * @return void
     */
    protected function _readFileCss( $file )
    {
      if ( file_exists( $file ) && filesize($file) > 0 ) 
      {
        $handle   = fopen( $file, "r" );
        $contents = fread( $handle, filesize($file) );
        fclose( $handle );

        $option = get_option( '__ipoptz_minify_css', 'false' );
        if ( $option == 'true' ) {
          $contents = $this->_minifyCss( $contents );
        }
        echo "$contents\n\n";
      }
    }

    //}}
    //{{ _minifyJs

    /**
     * Minify content
     *
     * @param  string $content
     * @return void
     */
    protected function _minifyJs( $content )
    {
      /*$path = IPOPTZ_PATH . 'app/libraries';
      require_once $path . '/minify/Minify.php';
      require_once $path . '/minify/CSS.php';
      require_once $path . '/minify/JS.php';
      require_once $path . '/minify/Exception.php';
      require_once $path . '/minify/Exceptions/BasicException.php';
      require_once $path . '/minify/Exceptions/FileImportException.php';
      require_once $path . '/minify/Exceptions/IOException.php';
      require_once $path . '/Converter.php';

      $minifier = new IPOptz_Minify_JS( $content );
      $content  = $minifier->minify();*/

      return $content;
    }

    //}}
    //{{ _minifyCss

    /**
     * Minify css content
     *
     * @param  string $content
     * @return void
     */
    protected function _minifyCss( $content )
    {
      $path = IPOPTZ_PATH . 'app/libraries';
      require_once $path . '/Converter.php';
      require_once $path . '/minify/Minify.php';
      require_once $path . '/minify/CSS.php';
      require_once $path . '/minify/JS.php';
      require_once $path . '/minify/Exception.php';
      require_once $path . '/minify/Exceptions/BasicException.php';
      require_once $path . '/minify/Exceptions/FileImportException.php';
      require_once $path . '/minify/Exceptions/IOException.php';

      $minifier = new IPOptz_Minify_CSS( $content );
      $content  = $minifier->minify();

      return $content;
    }

    //}}
    //{{ _show404

    /**
     * Move all scripts to footer
     *
     * @return void
     */
    protected function _show404()
    {
      global $wp_query;

      $wp_query->set_404();
      status_header(404);
    }

    //}}
  }
}